@@ -40,8 +40,8 @@
 defines
 *****************************************************************/
 
-#define SAB_VERSION "0.60"
-#define SAB_DATE "Jun 13, 2001"
+#define SAB_VERSION "0.65"
+#define SAB_DATE "Aug 15, 2001"
 #define BARE_SIGN '@'
 
 #define MAX_BARE        3
@@ -148,7 +148,7 @@ char usage[] =
 char askhelp[] = "Type sabcmd --help to display a help message.\n";
 char msgConflict[] = "conflict with preceding switches: ";
 char version_txt[] = "\nsabcmd "SAB_VERSION" ("SAB_DATE")\n"\
-    "copyright (C) 2000 Ginger Alliance (www.gingerall.com)\n";
+    "copyright (C) 2000, 2001 Ginger Alliance (www.gingerall.com)\n";
 char dbg_usage[] = "\nDebugging options:\n"\
     "\t--debug\t\t\tdisplay results of the command line parse\n"\
     "\t--times=COUNT, -t\trun sabcmd the specified number of times\n"\
