@@ -41,6 +41,8 @@
 #include "utf8.h"
 #include <locale.h>
 
+#include "guard.h"  // GP: clean
+
 /*****************************************************************
     language name aliases
     primarily for different language names in Windows setlocale
@@ -61,8 +63,12 @@ struct LangAlias
 
 LangAlias langAliases[] =
 {
-    {"cze ces cs cs_CZ",        "czech"},
-    {"sk slk slo",              "slovak"},
+    {"en",                      "en_US"},
+    {"de",                      "de_DE"},
+    {"fr",                      "fr_FR"},
+    {"it",                      "it_IT"},
+    {"cze ces cs cs_CZ",        "cs_CZ czech"},
+    {"sk slk slo sk_SK",        "sk_SK slovak"},
     {NULL, NULL}
 };
 
@@ -70,14 +76,20 @@ LangAlias langAliases[] =
     C L i s t   methods
 *****************************************************************/
 
-CList::CList(Processor *proc_)
+CList::CList()
 :
 VertexList(LIST_SIZE_LARGE)
 {
     sortDefs = NULL;
     refCount = 1;
     currLevel = 0;
-    proc = proc_;
+}
+
+// GP: added method to kill values
+
+CList::~CList()
+{
+    values.freeall(FALSE);
 }
 
 void CList::incRefCount()
@@ -173,8 +185,10 @@ char* setLang(const Str& src)
     return FALSE;
 }
 
-eFlag CList::sort(XSLElement *caller, Context *ctxt, SortDefList* sortDefs_ /* = NULL */)
+eFlag CList::sort(Sit S, XSLElement *caller, Context *ctxt, SortDefList* sortDefs_ /* = NULL */)
 {
+    // GP: on error, locale is reset
+    // checked that all callers destroy the context on error
     assert(caller || !sortDefs_);
     Str theLang;
     sortDefs = sortDefs_;
@@ -182,10 +196,12 @@ eFlag CList::sort(XSLElement *caller, Context *ctxt, SortDefList* sortDefs_ /* =
     {
         if (!setLang((*sortDefs)[0] -> lang))
         {
-            Warn1(proc->situation, W1_LANG, (*sortDefs)[0] -> lang);
+            Warn1(S, W1_LANG, (*sortDefs)[0] -> lang);
             setlocale(LC_COLLATE, "C");
         }
-        E( makeValues(0, number() - 1, 0, caller, ctxt) );
+        // this is to reset the locale if there's an error
+        E_DO( makeValues(S, 0, number() - 1, 0, caller, ctxt), 
+              setlocale(LC_COLLATE, "C") );
     }
     currLevel = 0;
     VertexList::sort(0, number() - 1);
@@ -193,7 +209,7 @@ eFlag CList::sort(XSLElement *caller, Context *ctxt, SortDefList* sortDefs_ /* =
     {
         if (!setLang((*sortDefs)[i] -> lang))
         {
-            Warn1(proc->situation, W1_LANG, (*sortDefs)[i] -> lang);
+            Warn1(S, W1_LANG, (*sortDefs)[i] -> lang);
             setlocale(LC_COLLATE, "C");
         }
         int j0 = 0;
@@ -203,7 +219,8 @@ eFlag CList::sort(XSLElement *caller, Context *ctxt, SortDefList* sortDefs_ /* =
                 if (j > j0 + 1)
                 {
                     currLevel = i;
-                    E( makeValues(j0, j-1, i, caller, ctxt));
+                    E_DO( makeValues(S, j0, j-1, i, caller, ctxt), 
+                          setlocale(LC_COLLATE, "C"));
                     // printf("Sorting %d:%d to %d\n",i,j0,j-1);
                     VertexList::sort(j0, j-1);
                     // reset currLevel for next compareWithoutDocOrd()
@@ -230,7 +247,17 @@ void CList::swap(int i, int j)
         values.swap(i, j);
 }
 
-eFlag CList::makeValues(int from, int to, int level, 
+void CList::append(Vertex *v)
+{
+    SList<Vertex*>::append(v);
+}
+
+void CList::rm(int ndx)
+{
+    SList<Vertex*>::rm(ndx);
+}
+
+eFlag CList::makeValues(Sit S, int from, int to, int level, 
                         XSLElement *caller, Context *ctxt)
 {
     assert(ctxt);
@@ -238,58 +265,67 @@ eFlag CList::makeValues(int from, int to, int level,
     assert(level < sortDefs -> number());
     SortDef *def = (*sortDefs)[level];
     DStr d;
-    Str *s;
-    Expression *e, result(caller, caller -> ownerT -> proc, EXF_ATOM);
+    GP( Str ) strg;
+    Expression *e, result(*caller, EXF_ATOM);
     for (int i = from; i <= to; i++)
     {
         ctxt -> setPosition(i);
-        s = new Str;
+        strg = new Str;
         e = def -> sortExpr;
         if (!e)
         {
-            E( operator[](i) -> value(d, ctxt) );
+            E( operator[](i) -> value(S, d, ctxt) );
             if (def -> asText)
 #               if defined(HAVE_WCSXFRM) || defined(_MSC_VER)
-                *s = utf8xfrm(d);
+                *strg = utf8xfrm(d);
 #               else
-                *s = d;
+                *strg = d;
 #               endif
             else
-                *s = d;
+                *strg = d;
         }
         else
         {
-            E( e -> eval(result, ctxt) );
+            E( e -> eval(S, result, ctxt) );
             if (def -> asText)
+	        {
 #               if defined(HAVE_WCSXFRM) || defined(_MSC_VER)
-                *s = utf8xfrm(result.tostring());
+                Str temp;
+		        E( result.tostring(S, temp) );
+                *strg = utf8xfrm(temp);
 #               else
-                *s = result.tostring();
+                E(result.tostring(S,*strg));
 #               endif
+            }
             else
-                *s = result.tostring();
+                E(result.tostring(S,*strg));
         }
         if (!level)
-            values.append(s);
+            values.append(strg.keep());
         else
         {
             cdelete(values[i]);
-            values[i] = s;
+            values[i] = strg.keep();
         }
     }
     return OK;
 }
 
+void CList::report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2)
+{
+    S.message(type, code, arg1, arg2);
+}
+
+
 /*****************************************************************
 
     C o n t e x t    methods
 
 ******************************************************************/
 
-Context::Context(Processor *proc_)
+Context::Context()
 {
-    proc = proc_;
-    array = new CList(proc);
+    array = new CList;
     currentNode = NULL;
     position = -1;
     virtualPosition = 0;
@@ -387,7 +423,7 @@ void Context::deppendall()
 {
     if (!array -> decRefCount())
         delete array;
-    array = new CList(proc);
+    array = new CList;
     position = -1;
 }
 
@@ -406,7 +442,7 @@ void Context::deppend()
 
 Context* Context::copy()
 {
-    Context *newc = new Context(proc);
+    Context *newc = new Context;
     delete NZ(newc -> array);
     newc -> currentNode = currentNode;
     newc -> array = array;
@@ -424,7 +460,7 @@ void Context::swap(int i, int j)
 
 Context *Context::swallow(Context *other)
 {
-    Context *result = new Context(proc);
+    Context *result = new Context; // GP: OK since no E()
     int i = 0, j = 0,
         iLimit = array -> number(), jLimit = other -> array -> number();
     long vstamp, wstamp;
@@ -477,3 +513,9 @@ void Context::uniquize()
         if (!array -> compare(i, i+1))
             array -> rm(i);
 }
+
+CList* Context::getArrayForDOM()
+{
+    return array;
+}
+
