@@ -33,6 +33,10 @@
 #include "datastr.h"
 #include "verts.h"
 #include "utf8.h"
+#include "tree.h"
+
+// GP: clean (only 1 eFlag routine)
+
 // #include <malloc.h> - not needed and deprecated anyway
 
 /* const Str* theEmptyString = new Str(""); */
@@ -351,6 +355,11 @@ Bool Str::operator== (char* otherChars) const
     return (Bool) !strcmp(operator char*(), otherChars);
 }
 
+Bool Str::eqNoCase (const Str& other) const
+{
+    return (strEqNoCase(operator char*(), (char*) other) ? TRUE : FALSE);
+}
+
 Bool Str::eqNoCase (const char* other) const
 {
     return (strEqNoCase(operator char*(), other) ? TRUE : FALSE);
@@ -628,9 +637,7 @@ void escapeChars(DStr& result, const Str& what,
 //
 //
 
-QName::QName(Processor *proc_)
-:
-proc( proc_ )
+QName::QName()
 {
     local = UNDEF_PHRASE;
     uri = UNDEF_PHRASE;
@@ -642,8 +649,14 @@ QName::QName(const QName &other)
     local = other.local;
     uri = other.uri;
     prefix = other.prefix;
+}
 
-    proc = other.proc;
+QName& QName::operator =(const QName& other)
+{
+    local = other.local;
+    uri = other.uri;
+    prefix = other.prefix;
+    return *this;
 }
 
 Bool QName::isEmpty() const
@@ -658,71 +671,44 @@ void QName::empty()
     uri = UNDEF_PHRASE;
 }
 
+/*
 void QName::speak(DStr &s, SpeakMode mode) const
 {
     if (prefix != UNDEF_PHRASE)
     {
-        s += proc -> dict().getKey(prefix);
+        s += dict().getKey(prefix);
         s += ":";
     };
-    s += proc -> dict().getKey(local);
+    s += dict().getKey(local);
 }
+*/
 
+/*
 Bool QName::prefixIsNS(const Str &uri2) const
 {
     Phrase uri2Id;
-    E( proc -> dict().insert(uri2, uri2Id) );
+    dict().insert(uri2, uri2Id);
     return (uri == uri2Id);
 } 
+*/
 
-eFlag QName::setLogical(const Str &s, const NSList *resolver, Bool defaultToo)
+void QName::setPrefix(Phrase prefix_)
 {
-    char *p = (char *)(Str &) s,
-        *q = strchr(p,':'),
-        *localPart;
-    if (q)
-    {
-        *q = 0;
-        setPrefix(p);
-        *q = ':';
-        localPart = q + 1;
-    }
-    else
-    {
-        prefix = UNDEF_PHRASE;
-        localPart = p;
-    };
-    E( NZ(resolver) -> resolve(uri = prefix, defaultToo) );
-    if (strchr(localPart,':'))
-        Err1(proc -> situation, E1_EXTRA_COLON, (char *)(Str &)s)
-    else
-        setLocal(localPart);
-    return OK;
-}
-
-void QName::setPrefix(const Str& pref)
-{
-    proc -> dict().insert(pref, prefix);
+    prefix = prefix_;
+//    dict().insert(pref, prefix);
 }
 
 
-void QName::setLocal(const Str& loc)
+void QName::setLocal(Phrase local_)
 {
-    proc -> dict().insert(loc, local);
+    local = local_;
+//    dict().insert(loc, local);
 }
 
-void QName::setUri(const Str& u)
+void QName::setUri(Phrase uri_)
 {
-    proc -> dict().insert(u, uri);
-}
-
-
-void QName::findPrefix(const NSList& resolver)
-{
-    if (uri == UNDEF_PHRASE)
-        prefix = UNDEF_PHRASE;
-    else
-        resolver.unresolve(prefix = uri);
+    uri = uri_;
+//    dict().insert(u, uri);
 }
 
 
@@ -731,33 +717,128 @@ Bool QName::operator== (const QName &other) const
     return local == other.local && uri == other.uri;
 }
 
+/*
 Str QName::getname() const
 {
     DStr s;
     speak(s,SM_NAME);
     return s;
 }
+*/
 
-const Str& QName::getLocal() const
+Phrase QName::getLocal() const
 {
-    return proc -> dict().getKey(local);
+    return local;
+//  return dict().getKey(local);
 }
 
-const Str& QName::getUri() const
+Phrase QName::getUri() const
 {
-    return proc -> dict().getKey(uri);
+    return uri;
+//  return dict().getKey(uri);
 }
 
-const Str& QName::getPrefix() const
+Phrase QName::getPrefix() const
 {
-    return proc -> dict().getKey(prefix);
+    return prefix;
+//  return dict().getKey(prefix);
 }
 
-const Bool QName::hasPrefix() const
+Bool QName::hasPrefix() const
 {
    return (prefix != UNDEF_PHRASE);
 }
 
+//
+//    EQName
+//
+
+EQName::EQName()
+{
+};
+
+EQName::EQName(const EQName& other)
+{
+    prefix = other.getPrefix();
+    uri = other.getUri();
+    local = other.getLocal();
+};
+
+Bool EQName::isEmpty() const
+{
+    return prefix.isEmpty() && uri.isEmpty() && local.isEmpty();
+}
+
+void EQName::empty()
+{
+    prefix.empty();
+    uri.empty();
+    local.empty();	    
+}
+
+void EQName::set(const QName& q, const HashTable& dict)
+{
+    prefix = dict.getKey(q.getPrefix());
+    uri = dict.getKey(q.getUri());
+    local = dict.getKey(q.getLocal());
+}
+
+void EQName::setPrefix(const Str& prefix_)
+{
+    prefix = prefix_;
+}
+
+void EQName::setLocal(const Str& local_)
+{
+    local = local_;
+}
+
+void EQName::setUri(const Str& uri_)
+{
+    uri = uri_;
+}
+
+void EQName::getname(Str& fullName) const
+{
+    DStr strg;
+    if (!prefix.isEmpty())
+    {
+        strg += prefix;
+        strg += ":";
+    };
+    strg += local;
+    fullName = strg;
+}
+
+const Str& EQName::getLocal() const
+{
+    return local;
+}
+
+const Str& EQName::getUri() const
+{
+    return uri;
+}
+
+const Str& EQName::getPrefix() const
+{
+    return prefix;
+}
+
+Bool EQName::hasPrefix() const
+{
+    return !prefix.isEmpty();
+}
+
+Bool EQName::operator==(const EQName &other) const
+{
+    return (uri == other.getUri() 
+        && local == other.getLocal() 
+	    && uri == other.getUri());
+}
+
+
+
 //
 //
 //                  StrStrList
@@ -800,7 +881,7 @@ int NamespaceStack::findNum(const Str &prefix_) const
     return i;
 }
 
-Str* NamespaceStack::getUri(const Str &_prefix) const
+const Str* NamespaceStack::getUri(const Str &_prefix) const
 {
     int ndx = findNum(_prefix);
     return (ndx != -1) ? &((*this)[ndx] -> uri) : NULL;
@@ -856,12 +937,12 @@ void P2List::appendConstruct(Phrase key, Phrase value)
 
 //
 //
-//                  QNameList
+//                  EQNameList
 //
 //
 
 
-const QName* QNameList::find(const QName &what) const
+const EQName* EQNameList::find(const EQName &what) const
 {
     int i, count = number();
     for (i = 0; (i < count) && !(what == (*operator[](i))); i++) {};
@@ -872,23 +953,27 @@ const QName* QNameList::find(const QName &what) const
 
 //
 //
-//                  QNameStrList
+//                  EQNameStrList
 //
 //
 
-
-const Str* QNameStrList::find(const QName &what) const
+// returns -1 if the EQName was not found
+int EQNameStrList::findNdx(const EQName &what) const
 {
     int i, count = number();
     for (i = 0; i < count && !(what == operator[](i) -> key); i++) {};
-    return (i < count) ? &(operator[](i) -> value) : NULL;
+    return (i < count) ? i : -1;
 }
 
-void QNameStrList::appendConstruct(const QName &key, const Str& value)
+const Str* EQNameStrList::find(const EQName &what) const
 {
-    QNameStr* newItem = new QNameStr(proc);
-    newItem -> key  = key;
-    newItem -> value = value;
+    int i = findNdx(what);
+    return (i != -1 ? &(operator[](i) -> value) : NULL);
+}
+
+void EQNameStrList::appendConstruct(const EQName &key, const Str& value)
+{
+    EQNameStr* newItem = new EQNameStr(key, value);
     append(newItem);
 }
 
