@@ -701,7 +701,8 @@ void LocStep::speak(Sit S, DStr &strg, SpeakMode mode)
     };
 };
 
-eFlag LocStep::parse(Sit S, Tokenizer& tokens, int& pos)
+eFlag LocStep::parse(Sit S, Tokenizer& tokens, int& pos, 
+    Bool defaultToo /* = FALSE */)
 {
     int right;
     int &i = pos;
@@ -773,8 +774,8 @@ eFlag LocStep::parse(Sit S, Tokenizer& tokens, int& pos)
         // set the QName from prefix:uri using
         // the namespace declarations in 'ownerE'
         // (passed through from the containing attribute)
-        // without using the default namespace
-        E( getOwnerElement().setLogical(S, ntest, temp, FALSE) ); 
+        // DEFAULT: without using the default namespace
+        E( getOwnerElement().setLogical(S, ntest, temp, defaultToo) ); 
 	}
 
     while ((tokens.items[i] -> tok) == TOK_LBRACKET)
@@ -1497,12 +1498,13 @@ eFlag Expression::patternOK(Sit S)
 }
 
 eFlag Expression::parse(Sit S, const DStr &string,
-                        Bool _isPattern /* = FALSE  */)
+                        Bool _isPattern /* = FALSE  */,
+			            Bool defaultToo /* = FALSE  */)
 {
     isPattern = _isPattern;
     Tokenizer t(*this);
     E( t.tokenize(S, string) );
-    E( parse(S, t, 0, t.items.number() - 1) );
+    E( parse(S, t, 0, t.items.number() - 1, defaultToo) );
     if (_isPattern)
         E( patternOK(S) );
     return OK;
@@ -1608,7 +1610,7 @@ eFlag parseLP()
 ================================================================*/
 
 eFlag Expression::parseLP(Sit S, Tokenizer& tokens, int &pos, 
-                     Bool dropRoot /*=FALSE*/)
+                     Bool dropRoot, Bool defaultToo /*=FALSE*/)
 {
     assert(functor == EXF_LOCPATH);
     ExToken tok;
@@ -1645,7 +1647,7 @@ eFlag Expression::parseLP(Sit S, Tokenizer& tokens, int &pos,
         case TOK_DPERIOD:
             {
                 args.append(ls = new Expression(getOwnerElement(),EXF_LOCSTEP));
-                E( ls -> step -> parse(S, tokens,i) );
+                E( ls -> step -> parse(S, tokens,i, defaultToo) );
                 slashPending = FALSE;
                 nameWas = TRUE;
             };
@@ -1686,7 +1688,8 @@ eFlag parseBasic()
     parentheses.
 ================================================================*/
 
-eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
+eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to,
+    Bool defaultToo /* = FALSE */)
 {
     GP( Expression ) e, lp;
     // find the start of the filtering predicates
@@ -1741,7 +1744,7 @@ eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
                     if (fleft + 1 == fright)
                         Err(S, ET_EXPR_SYNTAX);
                     E( (e = new Expression(getOwnerElement())) -> parse(S, t,
-                        fleft + 1, fright - 1) );
+                        fleft + 1, fright - 1, defaultToo) );
                     args.append(e.keep());
                     fleft = fright + 1;
                 };
@@ -1749,7 +1752,7 @@ eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
                     || (tok == TOK_DSLASH))
                 {
                     E( (lp = new Expression(getOwnerElement(), EXF_LOCPATH)) -> parseLP(
-                        S, t,fleft,TRUE) );
+                        S, t, fleft, TRUE, defaultToo) );
                     hasPath = TRUE;
                     args.append(lp.keep());
                 };
@@ -1828,7 +1831,7 @@ eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
                 };
                 args.append(e = new Expression(getOwnerElement()));
                 e.keep();
-                E( (*e).parse(S, t,i,j-1) );
+                E( (*e).parse(S, t,i,j-1, defaultToo) );
                 i = j+1;
             };
 
@@ -1840,7 +1843,7 @@ eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
             {
                 args.append(e = new Expression(getOwnerElement()));
                 e.keep();
-                E( (*e).parse(S, t,i,j-1) );
+                E( (*e).parse(S, t,i,j-1, defaultToo) );
             }
             if (to != j)
                 Err(S, ET_EXPR_SYNTAX);
@@ -1850,7 +1853,7 @@ eFlag Expression::parseBasic(Sit S, Tokenizer &t, int from, int to)
             type = EX_NODESET;
             functor = EXF_LOCPATH;
             int howfar = from;
-            E( parseLP(S, t, howfar) );
+            E( parseLP(S, t, howfar, FALSE, defaultToo) );
             if (howfar != to + 1)
                 Err(S, ET_EXPR_SYNTAX);
         }
@@ -1870,7 +1873,8 @@ INPUT
                 limits)
 ================================================================*/
 
-eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to)
+eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to,
+    Bool defaultToo /* = FALSE */)
 // isOp, skipParens
 {
     int i;
@@ -1921,7 +1925,7 @@ eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to)
     // leftmost points at its leftmost occurence
 
     if (minndx == -1)
-        E( parseBasic(S, t, from, to) )
+        E( parseBasic(S, t, from, to, defaultToo) )
     else 
     {
         int tablendx = t.items[minndx] -> tok - TOKGROUP_OPERATORS;
@@ -1938,19 +1942,19 @@ eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to)
             if (minndx != from)
                 Err(S, ET_EXPR_SYNTAX)
             else
-                E( e -> parse(S, t,from+1,to) );
+                E( e -> parse(S, t,from+1,to, defaultToo) );
             };
             break;
         case 2:
             {
-                E( e -> parse(S, t,from,minndx - 1) );
+                E( e -> parse(S, t,from,minndx - 1, defaultToo) );
                 args.append(e = new Expression(getOwnerElement())); // GP: OK
-                E( e -> parse(S, t,minndx + 1, to) );
+                E( e -> parse(S, t,minndx + 1, to, defaultToo) );
             };
             break;
         default:
             {
-                E( e -> parse(S, t,from,leftmost - 1) );
+                E( e -> parse(S, t,from,leftmost - 1, defaultToo) );
                 int another = leftmost, 
                     lastone = leftmost;
                 // tom 24-10-00
@@ -1961,7 +1965,7 @@ eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to)
                 while((another <= to) && (t.items[another]->tok != TOK_END))
                 {
                     args.append(e = new Expression(getOwnerElement())); // GP: OK
-                    E( e -> parse(S, t, lastone + 1, another - 1));
+                    E( e -> parse(S, t, lastone + 1, another - 1, defaultToo));
                     lastone = another;
 
                     // tom 14-11-00
@@ -1969,7 +1973,7 @@ eFlag Expression::parse(Sit S, Tokenizer& t, int from, int to)
                     another = t.findTop(t.items[another]->tok, another+1);
                 };
                 args.append(e = new Expression(getOwnerElement())); // GP: OK
-                E( e -> parse(S, t,lastone + 1, to) );
+                E( e -> parse(S, t,lastone + 1, to, defaultToo) );
             };
         };
     }
