@@ -109,6 +109,12 @@ const char* theURI_HTMLAtts[] =
     NULL
 };
 
+// tags new lines should not be inserted after by indent
+const char* theNoEolHTMLTags[] = 
+{
+    "img", NULL
+};
+
 Bool isEmptyHTMLTag(const Str& name)
 {
     int ndx = lookupNoCase(name, theEmptyHTML40Tags);
@@ -125,6 +131,13 @@ Bool isURI_HTMLAtt(const Str& name)
     return !!theURI_HTMLAtts[ndx];
 }
 
+Bool isNoEolHTMLTag(const Str& name)
+{
+    int ndx = lookupNoCase(name, theNoEolHTMLTags);
+    // return TRUE iff found
+    return !!theNoEolHTMLTags[ndx];
+}
+
 //
 //  OutputDefinition
 //
@@ -330,13 +343,17 @@ void OutputDefinition::report(Sit S, MsgType type, MsgCode code, const Str& arg1
 #define sendStr(SIT, STRING)     sendStrEsc(SIT, (STRING), ESCAPING_NONE)
 #define sendLit(SIT, LITERAL)    sendOut(SIT, (LITERAL), sizeof(LITERAL) - 1, ESCAPING_NONE)
 // S is the situation (existing in each context indentEOL is called in) 
-#define indentEOL() {if (indent) sendLit(S, "\n");}
+#define indentEOL() {sendLit(S, "\n");}
+#define indentSpace() {for (int i__ = 0; i__ < level; i__++) sendLit(S, "  ");}
+#define indentFull() {if (indent && after_markup) {indentEOL(); indentSpace();}}
 
 PhysicalOutputLayerObj::PhysicalOutputLayerObj()
 {
     curr = 0;
     encodingCD = (CDesc) -1;
     indent = FALSE;
+    after_markup = FALSE;  //pc
+    level = 0;  //pc
 }
 
 PhysicalOutputLayerObj::~PhysicalOutputLayerObj()
@@ -349,9 +366,9 @@ eFlag PhysicalOutputLayerObj::setOptions(Sit S,
     targetDataLine = targetDataLine_;
     outDef = outDef_;
     method = outDef -> getMethod();
-    indent = outDef -> getIndent();
     if (method != OUTPUT_UNKNOWN)
         E( outDef -> setDefaults(S) );
+    indent = outDef -> getIndent();
     //enc = outDef -> getEncoding();
     if (S.getProcessor())
         encoding = S.getProcessor() -> getHardEncoding();
@@ -395,7 +412,9 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
 {
     // begin output of start tag: output element name
     
-    indentEOL();
+    if (!isNoEolHTMLTag(name) || (method != OUTPUT_HTML && method != OUTPUT_XHTML)) 
+      indentFull();
+
     sendLit(S, "<");
     E( sendStr(S, name) );
 
@@ -446,12 +465,17 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
 
     // close the tag
 
+    after_markup = TRUE;  //pc
+
     if (!isEmpty)
-        sendLit(S, ">");
+    {
+      sendLit(S, ">");
+      level++;  //pc
+    }
     else
     {
         if (method == OUTPUT_HTML || method == OUTPUT_XHTML)
-        {
+	{
             if (!isEmptyHTMLTag(name))
             {
                 sendLit(S, "></");
@@ -462,9 +486,12 @@ eFlag PhysicalOutputLayerObj::outputElementStart(Sit S,
                 sendLit(S, ">");
             else
                 sendLit(S, " />");
+
+	    if (isNoEolHTMLTag(name)) 
+	        after_markup = FALSE;
         }
         else
-            sendLit(S, "/>");
+	    sendLit(S, "/>");
     };
     return OK;
 }
@@ -476,10 +503,12 @@ eFlag PhysicalOutputLayerObj::outputElementEnd(Sit S,
 {
     if (!isEmpty)
     {
-        indentEOL();
+	level--;  //pc
+        indentFull();
         sendLit(S, "</");
         E( sendStr(S, name) );
         sendLit(S, ">");
+	after_markup = TRUE;  //pc
     };
     return OK;
 }
@@ -487,7 +516,7 @@ eFlag PhysicalOutputLayerObj::outputElementEnd(Sit S,
 
 eFlag PhysicalOutputLayerObj::outputComment(Sit S, const Str& contents)
 {
-    indentEOL();
+    indentFull();
     sendLit(S, "<!--");
     const char *p = contents, *p_was = p;
     int len = contents.length();
@@ -504,6 +533,7 @@ eFlag PhysicalOutputLayerObj::outputComment(Sit S, const Str& contents)
     if (trailingHyphen)
         sendLit(S, " ");
     sendLit(S, "-->");
+    after_markup = TRUE;  //pc
     return OK;
 }
 
@@ -513,7 +543,7 @@ eFlag PhysicalOutputLayerObj::outputCDataSection(Sit S, const Str& contents)
     const char *p = contents, *p_was = p;
     if (!*p)
         return OK;
-	indentEOL();
+	indentFull();
     sendLit(S, "<![CDATA[");
     while (*p)
     {
@@ -525,20 +555,22 @@ eFlag PhysicalOutputLayerObj::outputCDataSection(Sit S, const Str& contents)
         }
     };
     sendLit(S, "]]>");
+    after_markup = TRUE;  //pc
     return OK;
 }
 
 eFlag PhysicalOutputLayerObj::outputPI(Sit S, const Str& target, const Str& data)
 {
-    indentEOL();
+    indentFull();
     sendLit(S, "<?");
     E( sendStr(S, target) );
     sendLit(S, " ");
     E( sendStr(S, data) );
-    if (method == OUTPUT_HTML)
+    if (method == OUTPUT_HTML && !(target == (const char*)"xml"))
         sendLit(S, ">");
     else
-        sendLit(S, "?>"); 
+        sendLit(S, "?>");
+    after_markup = TRUE;  //pc
     return OK;
 }
 
@@ -547,7 +579,7 @@ eFlag PhysicalOutputLayerObj::outputDTD(Sit S,
 {
     if (method == OUTPUT_TEXT)
         return OK;
-	indentEOL();
+    indentFull();
     sendLit(S, "<!DOCTYPE ");
     switch(method)
     {
@@ -593,6 +625,7 @@ eFlag PhysicalOutputLayerObj::outputDTD(Sit S,
         sendLit(S, ">");
 	else
         sendLit(S, ">\xa");
+    after_markup = TRUE;  //pc
     return OK;
 }
 
@@ -616,6 +649,7 @@ eFlag PhysicalOutputLayerObj::outputText(Sit S,
             E( sendStrEsc(S, contents, ESCAPING_NONE) );
         }; break;
     };
+    after_markup = FALSE;  //pc
     return OK;
 }
 
@@ -707,6 +741,23 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                     }; break;
                 }
             }; break;
+        case '>':
+            {
+                switch(escapeMode)
+                {
+                case ESCAPING_URI:
+                case ESCAPING_HTML_URI:
+                case ESCAPING_ATTR:
+                case ESCAPING_LT_AMP:
+                    {
+		                if (method == OUTPUT_HTML || method == OUTPUT_XHTML)
+				        {
+                            E( sendLit(S, "&gt;") );
+                            served = TRUE;
+			            }
+                    }; break;
+                }
+            }; break;
         case '&':
             {
                 switch(escapeMode)
@@ -788,6 +839,24 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                     }; break;
                 }
             }; break;
+	    default:
+	        {
+		        // escape non-ASCII values in URIs
+		        if (*data < 0)
+			    {
+                    switch(escapeMode)
+                    {
+                    case ESCAPING_URI:
+                    case ESCAPING_HTML_URI:
+                        {
+                            E( sendOut(S, smallBuf, 
+                                writeCharacterRef(smallBuf, data, escapeMode),
+                                ESCAPING_NONE) );
+                            served = TRUE;
+                        }; break;
+                    }
+			    }			        
+		    }
         };
 
 
@@ -826,7 +895,7 @@ eFlag PhysicalOutputLayerObj::sendOut(Sit S,
                         E( sendOut(S, smallBuf, destCharLen, ESCAPING_NONE) );
                         data += srcCharLen;
                         served = TRUE;
-                                            }
+			        }
                 }
             }
         }   // if (!served)
@@ -945,6 +1014,7 @@ eFlag OutputterObj::setOptions(Sit S,
         E( physical -> setOptions(S, targetDataLine_, outDef_) ); 
     };
     outDef = NZ(outDef_);
+    method = outDef -> getMethod(); //_PH_
     return OK;
 }
 
@@ -1057,6 +1127,18 @@ eFlag OutputterObj::eventBeginOutput(Sit S)
     return OK;
 }
 
+eFlag OutputterObj::eventBeginSubtree(Sit S)
+{
+    if (physical)
+    {
+        method = outDef -> getMethod();
+    }
+    // initialize the SAX interface
+    IF_SAX1( startDocument ); 
+    state = STATE_OUTSIDE;
+    return OK;
+}
+
 
 eFlag OutputterObj::eventElementStart(Sit S, const EQName& name)
 {
