@@ -116,6 +116,8 @@ eFlag TreeConstructer::parseDataLineUsingExpat(Sit S,
     XML_SetUserData(theParser, this);
     if (S.getProcessor())
         XML_SetBase(theParser, S.getProcessor() -> findBaseURI( t -> name ));
+	// wasn't this done before?
+	XML_SetParamEntityParsing(theParser, XML_PARAM_ENTITY_PARSING_ALWAYS);
 
     eFlag eCode = parseDataLineUsingGivenExpat(S, t, d, theParser);
 
@@ -161,9 +163,12 @@ int TreeConstructer::getCurrentLineNumber() const
   
 /* static */
 eFlag TreeConstructer::getDocEncoding(Sit S, const char *buf, Str& theEncoding, TreeConstructer *this_)
-// assumes the parse buffer null-terminated
+  // assumes the parse buffer null-terminated
 {
-    switch (*(const unsigned short*)buf)
+  // copy the unsigned shorts out of the string to avoid misalignment
+    unsigned short bom_part1, bom_part2;
+    memcpy (&bom_part1, buf, sizeof(unsigned short));
+    switch (bom_part1)
     {
     case 0xfeff:
     case 0xfffe:
@@ -172,22 +177,27 @@ eFlag TreeConstructer::getDocEncoding(Sit S, const char *buf, Str& theEncoding,
     case 0x003c:
     case 0x3c00:
         // no byte order mark!
-        if (((const unsigned short*)buf)[1] == 0x003f ||
-            ((const unsigned short*)buf)[1] == 0x3f00)
+        memcpy (&bom_part2, buf+1, sizeof(unsigned short));
+        if ((bom_part2 == 0x003f) ||
+            (bom_part2 == 0x3f00))
             theEncoding = "UTF-16";
         else
             this_ -> Warn(S, W_BAD_START);
         break;
     case 0x0000:
-        if (((const unsigned short*)buf)[1] == 0x003c ||
-            ((const unsigned short*)buf)[1] == 0x3c00 ) 
+        memcpy (&bom_part2, buf+1, sizeof(unsigned short));
+        if ((bom_part2 == 0x003c) ||
+            (bom_part2 == 0x3c00))
             theEncoding = "ISO-10646-UCS-4";
         else
             this_ -> Warn(S, W_BAD_START);
         break;
     default:
         {
-            if (buf[0] == '<' && buf[1] == '?' && buf[2] == 'x' && buf[3] == 'm')
+	// look for "<?xml " to distinguish from "<?xml-stylesheet"
+	// or any other similar processing instructions
+	  if (buf[0] == '<' && buf[1] == '?' && buf[2] == 'x' 
+	      && buf[3] == 'm' && buf[4] == 'l' && buf[5] == ' ')
             {
                 const char *p = buf;
                 p = strpbrk(p + 2,"=?");
