@@ -163,6 +163,7 @@ int SablotParse_(
     DStr base;
     *D = NULL;
     double time_was = getMillisecs();
+
     // make absolute address
     if (!buffer)
     {
@@ -170,9 +171,9 @@ int SablotParse_(
         base = Str("file://") + base;
         // should call Processor::findBaseURI() here (but have no Processor)
         makeAbsoluteURI(uri, base, absolute);
-	}
-	else
-	    absolute = "arg:/_parsed_";
+    }
+    else
+      absolute = "arg:/_parsed_";
     
     // create the tree and dataline
     DataLine line;
@@ -185,11 +186,15 @@ int SablotParse_(
 		    
     // open the line
     EE( line.open(SIT(S), absolute, DLMODE_READ, &argList) );
+    
     Log1(SIT(S), L1_PARSING, absolute);
     EE( tc.parseDataLineUsingExpat(SIT(S), newTree, &line) );        
     EE( line.close(SIT(S)) );
     *D = &(newTree.keep() -> getRoot());    
     Log1(SIT(S), L1_PARSE_DONE, getMillisecsDiff(time_was));
+
+    argList.freeall(false);
+
     return OK;
 }
 
