@@ -178,7 +178,7 @@ Declare
 
 Declare
 (
-    int SablotCreateSituation(SablotSituation *SP);
+    int SablotCreateSituation(SablotSituation *sPtr);
 )
 
 /* Sets situation flags. */
