@@ -676,7 +676,19 @@ SDOM_Exception SDOM_getAttributeList(SablotSituation s, SDOM_Node n, SDOM_NodeLi
 
 SDOM_Exception SDOM_docToString(SablotSituation s, SDOM_Document d, SDOM_char **pSerialized)
 {
-    toTree(d) -> serialize(SIT(s), *pSerialized);
+    // using explicit temp because of BeOS
+    char *serializedTemp = NULL;
+    toTree(d) -> serialize(SIT(s), serializedTemp);
+    *pSerialized = serializedTemp;
+    return SDOM_OK;
+}
+
+//_PH_
+SDOM_Exception SDOM_nodeToString(SablotSituation s, SDOM_Document d, 
+				 SDOM_Node n,
+				 SDOM_char **pSerialized)
+{
+    toTree(d) -> serializeNode(SIT(s), toE(n), *pSerialized);
     return SDOM_OK;
 }
 
@@ -713,13 +725,21 @@ SDOM_Exception SDOM_disposeNodeList(SablotSituation s, SDOM_NodeList l)
 SDOM_Exception SDOM_xql(SablotSituation s, const SDOM_char *query, SDOM_Node currentNode, SDOM_NodeList *pResult)
 {
     RootNode &root = toV(currentNode) -> getOwner().getRoot();
-    Expression queryEx(root);
+    
+    // find the document element
+    int i;
+    for (i = 0; i < root.contents.number(); i++)
+        if (isElement(root.contents[i])) break;
+	
+    Element &docElement = (i < root.contents.number()) ?
+        *toE(root.contents[i]) : root;
+    Expression queryEx(docElement);
     *pResult = NULL;
     
     // parse the query string as a non-pattern
-    if (queryEx.parse(SIT(s), (char*)query, FALSE))
+    if (queryEx.parse(SIT(s), (char*)query, FALSE, TRUE))
         SDOM_Err(s, SDOM_QUERY_PARSE_ERR);
-	
+		
 	// create an initial context
 	GP( Context ) dummy = new Context, c;
     c.assign(dummy);
