@@ -30,86 +30,119 @@
  * GPL.
  */
 
+#include <time.h>
 #include "base.h"
 #include "situa.h"
 #include "verts.h"
-#include <time.h>
+#include "encoding.h"
+#include "tree.h"
+
+// GP: clean
+
+SituaInfo& SituaInfo::operator=(const SituaInfo& other)
+{
+    pending = other.pending;
+    currV = other.currV;
+    currFile = other.currFile;
+    currMsg = other.currMsg;
+    currLine = other.currLine;
+    SDOMExceptionCode = other.SDOMExceptionCode;
+    return *this;
+}
+
+void SituaInfo::clear()
+{
+    pending = E_OK;
+    currV = NULL;
+    currFile.empty();
+    currMsg.empty();
+    currLine = 0;
+    SDOMExceptionCode = 0;
+}
+
 
 /*****************************************************************
 
-    S   i   t   u   a   t   i   o   n   O   b   j 
+    S   i   t   u   a   t   i   o   n 
 
 *****************************************************************/
 
-SituationObj::SituationObj(Processor* proc_) : proc ( proc_ )
+Situation::Situation()
 {
+    theRecoder = new Recoder;
     clear();
+    proc = NULL;
     logfile = errwfile = NULL;
     openDefaultFiles();
+    flags = 0;
 }
 
-SituationObj::~SituationObj()
+Situation::~Situation()
 {
+    theRecoder -> clear(*this);
+    cdelete(theRecoder);
     if (logfile)
         stdclose(logfile);
     if (errwfile)
         stdclose(errwfile);
 }
 
-Str SituationObj::timeStr()
+Recoder& Situation::recoder() const
+{
+    return *theRecoder;
+}
+
+void Situation::setFlag(SablotFlag f)
+{
+    flags |= f;
+}
+
+void Situation::resetFlag(SablotFlag f)
+{
+    flags &= ~f;
+}
+
+void Situation::setFlags(int f)
+{
+    flags = f;
+}
+
+Str Situation::timeStr()
 {
     time_t currtime;
     time(&currtime);
     return asctime(localtime(&currtime));
 }
 
-void SituationObj::pushCurrent()
+void Situation::setCurrV(Vertex *v)
 {
-    SitData *p = new SitData;
-    p -> currV = currV;
-    p -> currLine = currLine;
-    p -> currFile = currFile;
-    history.append(p);
+    info.currV = v;
+    info.currLine = v -> lineno;
 }
 
-void SituationObj::popCurrent()
-{
-    SitData *p = history.last();
-    currV = p -> currV;
-    currLine = p -> currLine;
-    currFile = p -> currFile;
-    history.freelast(FALSE);
-}
-
-void SituationObj::pushCurrV(Vertex *v)
-{
-    pushCurrent();
+void Situation::setCurrVDoc(Vertex *v)
+{    
     setCurrV(v);
+    info.currFile = v -> getOwner().name;
 }
 
-void SituationObj::setCurrV(Vertex *v)
+void Situation::setCurrLine(int lno)
 {
-    currV = v;
-    currLine = v -> lineno;
+    info.currLine = lno;
 }
 
-void SituationObj::setCurrLine(int lno)
+void Situation::setCurrFile(Str& fname)
 {
-    currLine = lno;
-}
-
-void SituationObj::setCurrFile(Str& fname)
-{
-    currFile = fname;
+    info.currFile = fname;
 };
 
-eFlag SituationObj::openDefaultFiles()
+eFlag Situation::openDefaultFiles()
 {
     E( msgOutputFile((char *) "stderr", NULL) );
     return OK;
 }
 
-eFlag SituationObj::closeFiles()
+eFlag Situation::closeFiles()
 {
     if (logfile)
         stdclose(logfile);
@@ -120,7 +153,7 @@ eFlag SituationObj::closeFiles()
     return OK;
 }
 
-eFlag SituationObj::eraseLog(char *newLogFile)
+eFlag Situation::eraseLog(char *newLogFile)
 {
     if (logfile)
         stdclose(logfile);
@@ -133,7 +166,7 @@ eFlag SituationObj::eraseLog(char *newLogFile)
     return OK;
 }
 
-eFlag SituationObj::msgOutputFile(char *_errwfn, char *_logfn)
+eFlag Situation::msgOutputFile(char *_errwfn, char *_logfn)
 {
     E( closeFiles() );
     if (_logfn)
@@ -165,13 +198,16 @@ char **constructMsgFields(PList<DStr*>& strings)
     return p;
 }
 
-void SituationObj::message(MsgType type, MsgCode code, 
-                         const Str& arg1, const Str& arg2)
+void Situation::generateMessage(MsgType type, MsgCode code, 
+                         const Str& arg1, const Str& arg2,
+			             Str& theMessage)
 {
     char buf[512];
     PList<DStr*> out;
     void *messengerUD;
-    MessageHandler *messenger = proc -> getMessageHandler(&messengerUD);
+    MessageHandler *messenger = NULL;
+    if (proc)
+        messenger = proc -> getMessageHandler(&messengerUD);
     if (messenger)
     {
         out.append(new DStr("msgtype:"));
@@ -189,23 +225,23 @@ void SituationObj::message(MsgType type, MsgCode code,
     }
     if (messenger)
         out.append(new DStr("module:Sablotron"));
-    if (!currFile.isEmpty())
+    if (!info.currFile.isEmpty())
     {
-        sprintf(buf,"URI:%s",(char*) currFile);
+        sprintf(buf,"URI:%s",(char*)(info.currFile));
         out.append(new DStr(buf));
     }
-    if (currLine && type != MT_LOG)
+    if (info.currLine && type != MT_LOG)
     {
-        sprintf(buf,"line:%d",currLine);
+        sprintf(buf,"line:%d",info.currLine);
         out.append(new DStr(buf));
     }
-    if (currV && type != MT_LOG)
+    if (info.currV && type != MT_LOG)
     {
         DStr nameStr;
-        currV -> speak(nameStr, SM_NAME);
+        info.currV -> speak(nameStr, SM_NAME);
         sprintf(buf,"node:%s%s'%s'",
-            vertexTypeNames[currV -> vt & VT_BASE],
-            (currV -> vt == VT_VERTEX ? "" : " "),
+            vertexTypeNames[info.currV -> vt & VT_BASE],
+            (info.currV -> vt == VT_VERTEX ? "" : " "),
             (char *) nameStr);
         out.append(new DStr(buf));
     }
@@ -219,14 +255,14 @@ void SituationObj::message(MsgType type, MsgCode code,
         out.append(new DStr(msgText));
     }
 
-    if (messenger)
+    if (messenger && !(flags & SAB_NO_ERROR_REPORTING))
     {
         // construct the message fields
         char **msgFields = constructMsgFields(out);
         MH_ERROR externalCode = 
             messenger -> makeCode(messengerUD, proc,
                 type == MT_ERROR ? 1 : 0, 
-				MH_FACILITY_SABLOTRON, (unsigned short)code);
+                                MH_FACILITY_SABLOTRON, (unsigned short)code);
         
         // FIXME: casting to MH_LEVEL -- necessary?
         switch(type)
@@ -244,89 +280,133 @@ void SituationObj::message(MsgType type, MsgCode code,
         delete[] msgFields;
         // note that the strings pointed at by msgFields members are deleted
         // upon destruction of 'out'
+    };
+
+    // in any case, construct the message and return it in theMessage
+        // construct the message
+    DStr fullout;
+    if (type != MT_LOG)
+    {
+        fullout = GetMessage((MsgCode)(MSG_ERROR + type)) -> text;
+        fullout += " ";
+        int outnum = out.number();
+        for (int j = 0; j < outnum; j++)
+        {
+            if (j < outnum - 1)
+                fullout += "[";
+            fullout += *out[j];
+            if (j < outnum - 1)
+                fullout += "] ";
+            if (j == outnum-2)
+                fullout += "\n  ";
+        }
     }
     else
     {
-        // construct the message
-        DStr fullout;
-        if (type != MT_LOG)
-        {
-            fullout = GetMessage((MsgCode)(MSG_ERROR + type)) -> text;
-            fullout += " ";
-            int outnum = out.number();
-            for (int j = 0; j < outnum; j++)
-            {
-                if (j < outnum - 1)
-                    fullout += "[";
-                fullout += *out[j];
-                if (j < outnum - 1)
-                    fullout += "] ";
-                if (j == outnum-2)
-                    fullout += "\n  ";
-            }
-        }
-        else
-        {
-            if (out.number())
-                fullout = *(out.last());
-        }
+        if (out.number())
+            fullout = *(out.last());
+    }
 
 
+    if (!messenger && !(type == MT_ERROR && (flags & SAB_NO_ERROR_REPORTING)))
+    {
         // display the message yourself
         FILE *thefile = (type == MT_LOG ? logfile : errwfile);
         if (thefile)
             fprintf(thefile,"%s\n",(char*) fullout);
-    };
+	}
+	
+	theMessage = fullout;
 
-    // in any case dispose of the temp string array
+    // dispose of the temp string array
     out.freeall(FALSE);
 };
 
 
-void SituationObj::error(MsgCode code, 
-                         const Str& arg1, const Str& arg2)
+void Situation::message(MsgType type, MsgCode code, 
+                 const Str& arg1, const Str& arg2)
 {
-    message(MT_ERROR, code, arg1, arg2);
-    // only log the error if we are using our own reporter
-    if (!proc -> getMessageHandler(NULL))
-        message(MT_LOG, code, arg1, arg2);
-    pending = code;
+    if (code == E2_SDOM)
+    {
+        infoDOM = info;
+	    info.clear();
+    }
+    else 
+    {
+        if (type == MT_ERROR)
+            infoDOM.clear();
+	}
+
+    Str temp;
+    if (type == MT_ERROR)
+        info.pending = code;
+    generateMessage(type, code, arg1, arg2, temp);
+    info.currMsg = temp;
+    // only log errors and warnings if we are using our own reporter
+    if ((type == MT_ERROR || type == MT_WARN) && 
+        (!proc || !proc -> getMessageHandler(NULL)))
+        generateMessage(MT_LOG, code, arg1, arg2, temp);
 }
 
-void SituationObj::warning(MsgCode code, 
-                         const Str& arg1, const Str& arg2)
+void Situation::report(Situation *sit, MsgType type, MsgCode code, const Str &str1, const Str &str2)
 {
-    message(MT_WARN, code, arg1, arg2);
-    // only log the warning if we are using our own reporter
-    if (!proc -> getMessageHandler(NULL))
-        message(MT_LOG, code, arg1, arg2);
+    message(type, code, str1, str2);
 }
 
-void SituationObj::logmsg(MsgCode code, 
-                         const Str& arg1, const Str& arg2)
-{
-    message(MT_LOG, code, arg1, arg2);
-}
 
-Bool SituationObj::isError()
+Bool Situation::isError()
 {
-    return (Bool) (pending != E_OK);
+    return (Bool) (info.pending != E_OK);
 };
 
-int SituationObj::getError() const
+int Situation::getError() const
 {
-    return pending;
+    return info.pending;
 }
 
-void SituationObj::clearError()
+void Situation::clearError()
 {
-    pending = E_OK;
+    info.pending = E_OK;
+    info.currMsg.empty();
 }
 
-void SituationObj::clear()
+void Situation::clear()
 {
-    currV = NULL;
-    currLine = 0;
-    currFile.empty();
-    clearError();
+    info.clear();
+    infoDOM.clear();
+}
+
+void Situation::setSDOMExceptionCode(int code)
+{
+    info.SDOMExceptionCode = code;
+}
+
+int Situation::getSDOMExceptionCode() const
+{
+    // the exception code was moved to infoDOM by message()
+    return infoDOM.SDOMExceptionCode;
+}
+
+void Situation::getSDOMExceptionExtra(MsgCode& theCode, 
+    Str& theMessage, Str& theDocument, int& theLine) const
+{
+    theCode = infoDOM.pending;
+    theMessage = infoDOM.currMsg;
+    theDocument = infoDOM.currFile;
+    theLine = infoDOM.currLine;    
+}
+
+void Situation::swapProcessor(void *& proc_)
+{
+    void *temp = proc;
+    proc = (Processor*) proc_;
+    proc_ = temp;
+}
+
+const Str& Situation::findBaseURI(const Str& unmappedBase)
+{
+    if (proc)
+        return proc -> findBaseURI(unmappedBase);
+	else
+	    return unmappedBase;
 }
