@@ -47,7 +47,8 @@ T r e e   methods
 ****************************************/
 
 Tree::Tree(const Str &aname, BOOL aXSLTree)
-: theArena(TREE_ARENA_SIZE), theDictionary(&theArena, TREE_DICT_LOGSIZE)
+: theArena(TREE_ARENA_SIZE), 
+  theDictionary(&theArena, TREE_DICT_LOGSIZE)
 {
     // theDictionary.initialize();
     QName &rootname = (QName&) getTheEmptyQName();
@@ -66,7 +67,7 @@ Tree::Tree(const Str &aname, BOOL aXSLTree)
 
 Tree::~Tree()
 {
-    // rulesList.freeall(FALSE); - done in ~RuleSList
+  // rulesList.freeall(FALSE); - done in ~RuleSList
     aliasesList.freeall(FALSE);
     getRoot().~RootNode();
     // this is just in case there's a processing error:
@@ -261,6 +262,25 @@ eFlag Tree::serialize(Sit S, char *& result)
     return OK;
 }
 
+eFlag Tree::serializeNode(Sit S, Element *v, char *& result)
+{
+    OutputterObj out;
+    OutputDefinition def;
+    GP( DataLine ) targetLine = new DataLine;
+    // set default options for output
+    EQName xmlMethod;
+    xmlMethod.setLocal((char*)"xml");
+    E( def.setItemEQName(S, XSLA_METHOD, xmlMethod, FALSE, NULL) );
+    E( def.setDefaults(S) ); 
+    E( (*targetLine).open(S, (const char*)"arg:/dummy_", DLMODE_WRITE, NULL) );
+    out.setOptions(S, targetLine, &def);
+    E( (*v).serializeSubtree(S, out) );
+    result = (*targetLine).getOutBuffer() -> compactToBuffer();
+    E( (*targetLine).close(S) );
+    targetLine.del();
+    return OK;
+}
+
 void Tree::makeStamps()
 {
     int stamp_ = 0;
@@ -358,9 +378,9 @@ eFlag Tree::processVertexAfterParse(Sit S, Vertex *v, TreeConstructer* tc)
                         {
                             QName q;
 			                EQName eq;
-					        expandQ(q, eq);
                             E( x -> setLogical(S, 
 			                    q, theAtt -> cont, FALSE) );
+					        expandQ(q, eq);
                             E( outputDef.setItemEQName(S, 
 			                    XSLA_METHOD, eq, TRUE, v) );
                         }; break;
