@@ -32,6 +32,9 @@
 
 #include "vars.h"
 #include "situa.h"
+#include "tree.h"
+
+// GP: clean
 
 #define iterate(VAR, LST) for(int VAR = 0; VAR < (LST).number(); VAR++)
 
@@ -78,11 +81,9 @@ VarBindings::~VarBindings()
     V a r s L i s t
 *****************************************************************/
 
-VarsList::VarsList(Processor* proc_)
-: SList<VarBindings*>(1)
+VarsList::VarsList(Tree& sheet_)
+: SList<VarBindings*>(1), sheet(sheet_)
 {
-    proc = proc_;
-    situation = proc->situation;
     currCallLevel = currNestLevel = 0;
 }
 
@@ -94,7 +95,7 @@ VarsList::~VarsList()
 VarBindings *VarsList::find(QName &name)
 {
     for (int i = 0; i < number(); i++)
-        if ((*this)[i] -> varname == name)
+        if (sheet.cmpQNames((*this)[i] -> varname,name))
             return (*this)[i];
     return NULL;
 }
@@ -104,9 +105,9 @@ int VarsList::compare(int first, int second)
     const QName 
         &q1 = (*this)[first] -> varname,
         &q2 = (*this)[second] -> varname;
-    int u = q1.getUri().compare(q2.getUri());
+    int u = sheet.expand(q1.getUri()).compare(sheet.expand(q2.getUri()));
     if (u) return u;
-    u = q1.getLocal().compare(q2.getLocal());
+    u = sheet.expand(q1.getLocal()).compare(sheet.expand(q2.getLocal()));
     return u;
 }
 
@@ -115,14 +116,13 @@ VarBindings *VarsList::getOrAdd(QName &q)
     VarBindings *record = find(q);
     if (!record)
     {
-        record = new VarBindings (proc);
-        record -> varname = q;
+        record = new VarBindings(q);
         insert(record);
     };
     return record;
 }
 
-eFlag VarsList::addBinding(QName& q, Expression *e, Bool force)
+eFlag VarsList::addBinding(Sit S, QName& q, Expression *e, Bool force)
 {
     VarBindings *record = getOrAdd(q);
     VarBindingItem *lastbind = NULL;
@@ -131,7 +131,7 @@ eFlag VarsList::addBinding(QName& q, Expression *e, Bool force)
     {
         lastbind = record -> bindings.last();
         if (lastbind -> callLevel == currCallLevel) // && !lastbind -> prebinding)
-            Err(situation, ET_MULT_ASSIGNMENT);
+            Err(S, ET_MULT_ASSIGNMENT);
     }
 
     /*
@@ -150,16 +150,16 @@ eFlag VarsList::addBinding(QName& q, Expression *e, Bool force)
         lastbind -> nestLevel == currNestLevel &&
         !force)
     {
-        it -> expr = new Expression(e -> ownerV,proc);
-        E( lastbind -> expr -> eval(*(it -> expr), NULL) );
-        delete e;
+        it -> expr = new Expression(e -> getOwnerElement()); // GP: OK
+        delete e; // moved above the following line
+        E( lastbind -> expr -> eval(S, *(it -> expr), NULL) );
     }
     else
         it -> expr = e;
     return OK;
 }
 
-eFlag VarsList::addPrebinding(QName &q, Expression *e)
+eFlag VarsList::addPrebinding(Sit S, QName &q, Expression *e)
 {
     VarBindings *record = getOrAdd(q);
     VarBindingItem *lastbind = NULL;
@@ -170,7 +170,7 @@ eFlag VarsList::addPrebinding(QName &q, Expression *e)
         if (lastbind -> nestLevel == currNestLevel && 
             lastbind -> callLevel == currCallLevel &&    // new
             lastbind -> prebinding)
-            Err(situation, ET_MULT_ASSIGNMENT);
+            Err(S, ET_MULT_ASSIGNMENT);
     }
 
     VarBindingItem *it = new VarBindingItem;
@@ -294,3 +294,7 @@ void VarsList::rmPrebindings()
     }
 }
 
+void VarsList::report(Sit S, MsgType type, MsgCode code, const Str& arg1, const Str& arg2)
+{
+    S.message(type,code,arg1,arg2);
+}
