@@ -633,6 +633,16 @@ void RootNode::speak(DStr &s, SpeakMode mode)
         Daddy::speak(s,mode);
 }
 
+eFlag RootNode::startCopy(Sit S, OutputterObj& out)
+{
+    return Vertex::startCopy(S, out);
+};
+
+eFlag RootNode::endCopy(Sit S, OutputterObj& out)
+{
+    return Vertex::endCopy(S, out);
+};
+
 eFlag RootNode::copy(Sit S, OutputterObj& out)
 {
     E( startCopy(S, out) );
@@ -816,7 +826,7 @@ eFlag Element::setLogical(Sit S, QName &qn, const Str &string, Bool defaultToo)
     E( namespaces.resolve(S, resolved, defaultToo) );
     // check if a non-undef was resolved to an undef
     if (resolved == UNDEF_PHRASE && thePrefix != UNDEF_PHRASE)
-        Err1(S, ET_BAD_PREFIX, (char *)(Str &)string);
+      Err1(S, ET_BAD_PREFIX, (char *)(Str &)string);
     if (strchr(localPart,':'))
         Err1(S, E1_EXTRA_COLON, (char *)(Str &)string);
     qn.setUri(resolved);
@@ -835,6 +845,21 @@ eFlag Element::serialize(Sit S, OutputterObj &out)
     return OK;
 }
 
+eFlag Element::serializeSubtree(Sit S, OutputterObj &out)
+{
+    E( out.eventBeginSubtree(S) );
+
+    EQName ename;
+    getOwner().expandQ(name, ename);
+    E( out.eventElementStart(S, ename) );
+    E( atts.serialize(S, out) );
+    E( contents.serialize(S, out) );
+    E( out.eventElementEnd(S, ename) );
+
+    E( out.eventEndOutput(S) );
+    return OK;
+}
+
 void Element::removeChild(Vertex *child)
 {
     assert(child -> parent == this);
